@@ -23,10 +23,9 @@
    @author Mark Seligman
  */
 
-#include "callback.h"
-#include <R.h>
 
 #include "rcppSample.h"
+#include "callback.h"
 
 /**
    @brief Initializes static state parameters for row sampling.
@@ -39,7 +38,7 @@
 
    @return void.
  */
-void CallBack::SampleInit(unsigned int _nRow, double _weight[], bool _repl) {
+void CallBack::SampleInit(unsigned int _nRow, const double _weight[], bool _repl) {
   RcppSample::Init(_nRow, _weight, _repl);
 }
 
@@ -58,42 +57,6 @@ void CallBack::SampleRows(unsigned int nSamp, int out[]) {
 }
 
 
-/**
-  @brief Call-back to R's integer quicksort with indices.
-
-  @param ySorted[] is a copy-out vector containing the sorted integers.
-
-  @param rank2Row[] is the vector of permuted indices.
-
-  @param one is a hard-coded integer indicating unit stride.
-
-  @param nRow is the number of rows to sort.
-
-  @return Formally void, with copy-out parameter vectors.
-*/
-void CallBack::QSortI(int ySorted[],  int rank2Row[], int one,  int nRow) {
-  R_qsort_int_I(ySorted, (int *) rank2Row, one, (int) nRow);
-}
-
-
-/**
-   @brief Call-back to R's double quicksort with indices.
-
-   @param ySorted[] is the copy-out vector of sorted values.
-
-   @param rank2Row[] is the copy-out vector of permuted indices.
-
-   @param one is a hard-coded integer indicating unit stride.
-
-   @param nRow is the number of rows to sort.
-
-   @return Formally void, with copy-out parameter vectors.
-*/
-void CallBack::QSortD(double ySorted[],  int rank2Row[], int one,  int nRow) {
-  R_qsort_I(ySorted, (int*) rank2Row, one, (int) nRow);
-}
-
-
 /**
    @brief Call-back to R's uniform random-variate generator.
 
@@ -111,3 +74,45 @@ void CallBack::RUnif(int len, double out[]) {
   for (int i = 0; i < len; i++)
     out[i] = rn[i];
 }
+
+
+// Leaving front-end sorts in place for now, as std:: versions
+// not offering a performance advantage, even when run in parallel:
+//
+
+/**
+  @brief Call-back to R's integer quicksort with indices.
+
+  @param ySorted[] is a copy-out vector containing the sorted integers.
+
+  @param rank2Row[] is the vector of permuted indices.
+
+  @param one is a hard-coded integer indicating unit stride.
+
+  @param nRow is the number of rows to sort.
+
+  @return Formally void, with copy-out parameter vectors.
+*/
+void CallBack::QSortI(int ySorted[],  unsigned int rank2Row[], int one,  int nRow) {
+  R_qsort_int_I(ySorted, (int *) rank2Row, one, (int) nRow);
+}
+
+
+/**
+   @brief Call-back to R's double quicksort with indices.
+
+   @param ySorted[] is the copy-out vector of sorted values.
+
+   @param rank2Row[] is the copy-out vector of permuted indices.
+
+   @param one is a hard-coded integer indicating unit stride.
+
+   @param nRow is the number of rows to sort.
+
+   @return Formally void, with copy-out parameter vectors.
+*/
+void CallBack::QSortD(double ySorted[],  unsigned int rank2Row[], int one,  int nRow) {
+  R_qsort_I(ySorted, (int*) rank2Row, one, (int) nRow);
+}
+
+
