@@ -28,11 +28,11 @@
 
 class CallBack {
  public:
-  static void SampleInit(unsigned int _nRow, double _sampleWeight[], bool _withRepl);
+  static void SampleInit(unsigned int _nRow, const double _sampleWeight[], bool _withRepl);
   static void SampleRows(unsigned int nSamp, int out[]);
-  static void QSortI(int ySorted[],  int rank2Row[], int one,  int nRow);
-  static void QSortD(double ySorted[],  int rank2Row[], int one,  int nRow);
   static void RUnif(int len, double out[]);
+  static void QSortI(int ySorted[], unsigned int rank2Row[], int one, int nRow);
+  static void QSortD(double ySorted[], unsigned int rank2Row[], int one, int nRow);
 };
 
 #endif
