@@ -17,6 +17,8 @@
 #ifndef ARBORIST_INDEX_H
 #define ARBORIST_INDEX_H
 
+#include <vector>
+
 /**
    Index tree node fields associated with the response, viz., invariant across
    predictors.  IndexNodes of the index tree can be thought of as representing
@@ -29,13 +31,15 @@
 class IndexNode {
   double preBias; // Inf of information values eligible for splitting.
  public: // The three integer values are all non-negative.
-  int splitIdx; // Position within containing vector:  split index.
-  int lhStart; // Start index of LHS data in buffer.
+  IndexNode();
+  unsigned int splitIdx; // Position within containing vector:  split index.
+  unsigned int lhStart; // Start index of LHS data in buffer.
   unsigned int idxCount; // # distinct indices in the node.
   unsigned int sCount;  // # samples subsumed by this node.
   double sum; // Sum of all responses in node.
   double minInfo; // Minimum acceptable information on which to split.
   unsigned int ptId; // Index of associated PTSerial node.
+  unsigned char path; // Bitwise record of recent reaching L/R path.
 
   double PrebiasReg();
   double PrebiasCtg(const double sumSquares[]);
@@ -84,20 +88,26 @@ class IndexNode {
 
      @return void.
   */
-  void Init(int _splitIdx, unsigned int _ptId, int _idxCount, unsigned int _sCount, double _sum, double _minInfo) {
+  void Init(int _splitIdx, unsigned int _start, unsigned int _ptId, int _idxCount, unsigned int _sCount, double _sum, double _minInfo, unsigned char _path) {
     splitIdx = _splitIdx;
     ptId = _ptId;
+    lhStart = _start;
     idxCount = _idxCount;
     sCount = _sCount;
     sum = _sum;
     minInfo = _minInfo;
-    lhStart = 0;
+    path = _path;
   }
 
+  inline void PathCoords(unsigned int &_start, unsigned int &_extent) {
+    _start = lhStart;
+    _extent = idxCount;
+  }
+  
   /**
      @return reference to 'lhStart' field.
    */
-  inline int &Start() {
+  inline unsigned int &Start() {
     return lhStart;
   }
 
@@ -118,7 +128,7 @@ class IndexNode {
 
      @return preBias, with output parameters.
   */
-  double inline SplitFields(int &_lhStart, int &_end, unsigned int &_sCount, double &_sum) const {
+  double inline SplitFields(unsigned int &_lhStart, unsigned int &_end, unsigned int &_sCount, double &_sum) const {
     _lhStart = lhStart;
     _end = _lhStart + idxCount - 1;
     _sCount = sCount;
@@ -126,57 +136,64 @@ class IndexNode {
     return preBias;
   }
 
-  void inline Extent(int &_lhStart, int &_end) const {
+  void inline Extent(unsigned int &_lhStart, unsigned int &_end) const {
     _lhStart = lhStart;
     _end = lhStart + idxCount - 1;
   }
 
-};
-
-
-/**
-   @brief Caches intermediate IndexNode contents during intra-level transfer.
-*/
-class NodeCache : public IndexNode {
-  class SSNode *ssNode; // Convenient to cache for LH/RH partition.
-  static unsigned int minNode;
-  unsigned int ptL; // LH index into pre-tree:  splits only.
-  unsigned int ptR; // RH index into pre-tree:  splits only.
- public:
-  static void Immutables(unsigned int _minNode);
-  static void DeImmutables();
-  void Consume(class Index *index, class PreTree *preTree, class SplitPred *splitPred, class SamplePred *samplePred, class RestageMap *restageMap, unsigned int level, int lhSplitNext, int &lhCount, int &rhCount);
-  void SplitCensus(int &lhSplitNext, int &rhSplitNext, int &leafNext) const;
-
-  /**
-     @brief Copies indexNode entry into corresponding nodeCache.
-
-     @param nd is the IndexNode to copy.
-
-     @return void.
-   */
-  inline void Cache(IndexNode *nd) {
-    splitIdx = nd->splitIdx;
-    lhStart = nd->lhStart;
-    idxCount = nd->idxCount;
-    sCount = nd->sCount;
-    sum = nd->sum;
-    ptId = nd->ptId;
-    minInfo = nd->minInfo;
-    ptL = ptR = 0; // Terminal until shown otherwise.
-  }
-
 
   /**
      @brief Exposes minimum-information value for the node.
 
      @return minInfo value.
    */
-  inline double MinInfo() {
+  inline double MinInfo() const {
     return minInfo;
   }
+};
+
+
+/**
+   @brief Caches intermediate IndexNode contents during intra-level transfer.
+*/
+class NodeCache : public IndexNode {
+  class SSNode *ssNode; // Convenient to cache for LH/RH partition.
+  bool terminal; // True unless next-level descendants produced.
+  static unsigned int minNode;
+  unsigned int lhIdxCount; // Total indices over LH:  splits only.
+  unsigned int lhSCount; // Total samples cover LH:  splits only.
+  double lhSum; // Sum of responses over LH:  splits only.
+  unsigned int ptL; // LH index into pre-tree:  splits only.
+  unsigned int ptR; // RH index into pre-tree:  splits only.
+ public:
+  static void Immutables(unsigned int _minNode);
+  static void DeImmutables();
+  NodeCache();
+  void Consume(class PreTree *preTree, class SamplePred *samplePred, class Bottom *bottom);
+  void Successors(class Index *index, class PreTree *preTree, class SamplePred *samplePred, class Bottom *bottom, unsigned int lhSplitNext, unsigned int &lhCount, unsigned int &rhCount);
+  void SplitCensus(unsigned int &lhSplitNext, unsigned int &rhSplitNext, unsigned int &leafNext);
+
+  /**
+     @brief Copies indexNode entry into corresponding nodeCache.
+
+     @param nd is the IndexNode to copy.
+
+     @return void.
+   */
+  inline void Cache(IndexNode *nd, class SSNode *argMax) {
+    splitIdx = nd->splitIdx;
+    lhStart = nd->lhStart;
+    idxCount = nd->idxCount;
+    sCount = nd->sCount;
+    sum = nd->sum;
+    ptId = nd->ptId;
+    minInfo = nd->minInfo;
+    path = nd->path,
+    ptL = ptR = 0; // Terminal until shown otherwise.
+    SS() = argMax;
+  }
+
 
-  
   inline class SSNode *&SS() {
     return ssNode;
   }
@@ -198,43 +215,58 @@ class NodeCache : public IndexNode {
   }
 
 
+  
+  /**
+     @brief Transmits location coordinates for interlevel activity.
+
+     @param _start outputs starting index position.
+
+     @param _extent output index count.
+
+     @return true iff no successors produced.
+   */
+  inline bool Terminal(unsigned int &_start, unsigned int &_extent, unsigned int &_lhIdxCount) const {
+    _start = lhStart;
+    _extent = idxCount;
+    _lhIdxCount = lhIdxCount; // nonzero iff nonterminal.
+
+    return terminal;
+  }
+  
 };
 
 class Index {
   static unsigned int totLevels;
-  NodeCache *CacheNodes();
-  void ArgMax(NodeCache nodeCache[], const class SplitSig *splitSig);
-  int LevelCensus(NodeCache nodeCache[], int &lhSplitNext, int &leafNext);
-  class RestageMap *ProduceNext(NodeCache nodeCache[], int splitNext, int lhSplitNext, int leafNext, unsigned int level);
+  NodeCache *CacheNodes(const std::vector<class SSNode*> &argMax);
+  void ArgMax(NodeCache nodeCache[]);
+  unsigned int LevelCensus(NodeCache nodeCache[], unsigned int levelCount, unsigned int &lhSplitNext, unsigned int &leafNext);
+  NodeCache *LevelConsume(unsigned int levelCount, unsigned int &splitNext, unsigned int &lhSplitNext, unsigned int &leafNext);
+  void LevelProduce(NodeCache *nodeCache, unsigned int level, unsigned int levelCount, unsigned int splitNext, unsigned int lhSplitNext, unsigned int leafNext);
  protected:
-  unsigned int level; // Zero-based level number.
-  int splitCount; // Width of current level.
   IndexNode *indexNode;  
   const unsigned int bagCount;
   unsigned int levelBase; // Pre-tree index at which level's nodes begin.
   unsigned int levelWidth; // Count of pretree nodes at frontier.
   bool *ntLH;
   bool *ntRH;
-  static class PreTree *OneTree(class SamplePred *_samplePred, class SplitPred *_splitPred, int _nSamp, int _bagCount, double _bagSum);
+  static class PreTree *OneTree(class SamplePred *_samplePred, class Bottom *_bottom, int _nSamp, int _bagCount, double _bagSum);
+
  public:
   static void Immutables(unsigned int _minNode, unsigned int _totLevels);
   static void DeImmutables();
   class SamplePred *samplePred;
   class PreTree *preTree;
-  class SplitPred *splitPred;
-  Index(class SamplePred *_samplePred, class PreTree *_preTree, class SplitPred *_splitPred, int _nSamp, int _bagCount, double _sum);
+  class Bottom *bottom;
+  Index(class SamplePred *_samplePred, class PreTree *_preTree, class Bottom *_bottom, int _nSamp, int _bagCount, double _sum);
   ~Index();
-  /**
-     @brief level accessor.
-   */
-  unsigned int Level() {
-    return level;
-  }
+
   static class PreTree **BlockTrees(class Sample **sampleBlock, int _treeBlock);
   void SetPrebias();
   void Levels();
   void PredicateBits(class BV *bitsLH, class BV *bitsRH, int &lhIdxTot, int &rhIdxTot) const;
+  void LRLive(class Bottom *bottom, const NodeCache *nodeCache, unsigned int level) const;
 
+  
   /**
      @brief 'bagCount' accessor.
 
@@ -260,20 +292,24 @@ class Index {
   }
 
 
-  inline void NextLH(int idxNext, unsigned int ptId, int idxCount, unsigned int sCount, double sum, double minInfo) {
-    indexNode[idxNext].Init(idxNext, ptId, idxCount, sCount, sum, minInfo);
+  inline unsigned int NextLH(int idxNext, unsigned int ptId, unsigned int _start, int idxCount, unsigned int sCount, double sum, double minInfo, unsigned char _path) {
+    unsigned int pathNext = _path << 1;
+    indexNode[idxNext].Init(idxNext, _start, ptId, idxCount, sCount, sum, minInfo, pathNext);
 
     SetLH(ptId);
+    return pathNext;
   }
 
   
-  inline void NextRH(int idxNext, int ptId, int idxCount, unsigned int sCount, double sum, double minInfo) {
-    indexNode[idxNext].Init(idxNext, ptId, idxCount, sCount, sum, minInfo);
+  inline unsigned int NextRH(int idxNext, int ptId, unsigned int _start, int idxCount, unsigned int sCount, double sum, double minInfo, unsigned char _path) {
+    unsigned int pathNext = (_path << 1) | 1;
+    indexNode[idxNext].Init(idxNext, _start, ptId, idxCount, sCount, sum, minInfo, pathNext);
 
     SetRH(ptId);
+    return pathNext;
   }
 
-  
+
   /**
      @brief Computes a level-relative offset for an indexed Pretree node.
 
