@@ -26,7 +26,6 @@
 class Quant {
   const class PredictReg *predictReg;
   const class LeafReg *leafReg;
-  const std::vector<double> &yRanked;
   const std::vector<double> &qVec;
   std::vector<unsigned int> sampleOffset;
   const unsigned int qCount;
@@ -42,7 +41,7 @@ class Quant {
   unsigned int RanksExact(unsigned int tIdx, unsigned int leafIdx, unsigned int sampRanks[]);
   unsigned int RanksSmudge(unsigned int tIdx, unsigned int LeafIdx, unsigned int sampRanks[]);
  public:
-  Quant(const class PredictReg *_predictReg, const class LeafReg *_leafReg, const std::vector<double> &_yRanked, const std::vector<double> &_qVec, unsigned int qBin);
+  Quant(const class PredictReg *_predictReg, const class LeafReg *_leafReg, const std::vector<double> &_qVec, unsigned int qBin);
   ~Quant();
   void PredictAcross(unsigned int rowStart, unsigned int rowEnd, double qPred[]);
 };
